@@ -1,7 +1,7 @@
 #!/usr/bin/python
 
 #
-# Copyright 2016 David Steele <steele@debian.org>
+# Copyright 2016-2017 David Steele <steele@debian.org>
 # This file is part of comitup
 # Available under the terms of the GNU General Public License version 2
 # or later
@@ -22,6 +22,7 @@ if __name__ == '__main__':
 import nmmon    # noqa
 import nm       # noqa
 import mdns     # noqa
+import modemgr  # noqa
 
 
 log = logging.getLogger('comitup')
@@ -35,12 +36,13 @@ com_state = None
 conn_list = []
 connection = ''
 state_id = 0
-last_activity = 0
 
 points = []
 
 state_callbacks = []
 
+hotspot_name = None
+
 
 def state_callback(fn):
     @wraps(fn)
@@ -62,8 +64,7 @@ def timeout(fn):
     @wraps(fn)
     def wrapper(id):
         if id == state_id:
-            if time.time() - last_activity > 60:
-                fn()
+            fn()
             return True
         else:
             return False
@@ -82,23 +83,47 @@ def dns_to_conn(host):
 # Hotspot state
 #
 
+def fake_hs_pass():
+    hotspot_pass()
+    return False
+
 
 @state_callback
 def hotspot_start():
     global conn_list
     log.info("Activating hotspot")
 
-    mdns.clear_entries()
-    conn_list = []
-    activate_connection(dns_to_conn(dns_names[0]))
+    hs_ssid = dns_to_conn(dns_names[0])
+
+    # if we are in two-wifi device mode, skip the reconnect if possible,
+    # to avoid kicking some clients off
+    if hs_ssid != nm.get_active_ssid(modemgr.get_state_device('HOTSPOT')):
+        mdns.clear_entries()
+        conn_list = []
+
+        # tolerate Raspberry Pi 2
+        try:
+            activate_connection(hs_ssid, 'HOTSPOT')
+        except DBusException:
+            log.warn("Error connecting hotspot")
+    else:
+        log.debug("Didn't need to reactivate - already running")
+        # the connect callback won't happen - let's 'pass' manually
+        gobject.timeout_add(100, fake_hs_pass)
 
 
 @state_callback
 def hotspot_pass():
     log.debug("Activating mdns")
 
-    ip = nm.get_active_ip()
-    mdns.add_hosts(dns_names, ip)
+    # IP tolerance for PI 2
+    for _ in range(5):
+        ip = nm.get_active_ip(modemgr.get_state_device('HOTSPOT'))
+        if ip:
+            mdns.clear_entries()
+            mdns.add_hosts(dns_names)
+            break
+        time.sleep(1)
 
 
 @state_callback
@@ -109,10 +134,11 @@ def hotspot_fail():
 @timeout
 def hotspot_timeout():
 
-    if iwscan.ap_conn_count() == 0:
+    if iwscan.ap_conn_count() == 0 or modemgr.get_mode() != 'single':
         log.debug('Periodic connection attempt')
 
-        conn_list = candidate_connections()
+        dev = modemgr.get_state_device('CONNECTED')
+        conn_list = candidate_connections(dev)
         if conn_list:
             # bug - try the first connection twice
             set_state('CONNECTING', [conn_list[0], conn_list[0]] + conn_list)
@@ -134,15 +160,16 @@ def connecting_start():
     mdns.clear_entries()
 
     if conn_list:
-        nm.disconnect()
+        nm.disconnect(modemgr.get_state_device('CONNECTING'))
 
         conn = conn_list.pop(0)
         log.info('Attempting connection to %s' % conn)
-        activate_connection(conn)
+        activate_connection(conn, 'CONNECTING')
     else:
         # Give NetworkManager a chance to update the access point list
         try:
-            nm.deactivate_connection()  # todo - clean this up
+            # todo - clean this up
+            nm.deactivate_connection(modemgr.get_state_device('CONNECTING'))
         except DBusException:
             pass
         time.sleep(5)
@@ -178,8 +205,14 @@ def connecting_timeout():
 def connected_start():
     global conn_list
 
-    ip = nm.get_active_ip()
-    mdns.add_hosts(dns_names, ip)
+    # IP tolerance for PI 2
+    for _ in range(5):
+        ip = nm.get_active_ip(modemgr.get_state_device('CONNECTED'))
+        if ip:
+            mdns.clear_entries()
+            mdns.add_hosts(dns_names)
+            break
+        time.sleep(1)
 
     conn_list = []
 
@@ -197,7 +230,7 @@ def connected_fail():
 
 @timeout
 def connected_timeout():
-    if connection != nm.get_active_ssid():
+    if connection != nm.get_active_ssid(modemgr.get_state_device('CONNECTED')):
         log.warn("Connection lost on timeout")
         set_state('HOTSPOT')
 
@@ -222,17 +255,17 @@ class state_matrix(object):
             raise AttributeError
 
 
-def set_state(state, connections=None):
+def set_state(state, connections=None, timeout=180):
     global com_state, conn_list, state_id, points
 
     log.info('Setting state to %s' % state)
 
     if com_state != 'HOTSPOT':
-        points = nm.get_points_ext()
+        points = nm.get_points_ext(modemgr.get_state_device(com_state))
 
     state_info = state_matrix(state)
 
-    nmmon.set_device_callbacks(state_info.pass_fn, state_info.fail_fn)
+    nmmon.set_device_callbacks(state, state_info.pass_fn, state_info.fail_fn)
 
     if connections:
         conn_list = connections
@@ -241,16 +274,10 @@ def set_state(state, connections=None):
     com_state = state
     state_info.start_fn()
 
-    gobject.timeout_add(60*1000, state_info.timeout_fn, state_id)
+    gobject.timeout_add(timeout*1000, state_info.timeout_fn, state_id)
 
 
-def set_activity():
-    global last_activity
-
-    last_activity = time.time()
-
-
-def activate_connection(name):
+def activate_connection(name, state):
     global connection
     connection = name
     log.debug('Connecting to %s' % connection)
@@ -260,11 +287,13 @@ def activate_connection(name):
     except IndexError:
         path = '/'
 
-    nm.activate_connection_by_ssid(connection, path=path)
+    nm.activate_connection_by_ssid(connection,
+                                   modemgr.get_state_device(state),
+                                   path=path)
 
 
-def candidate_connections():
-    return nm.get_candidate_connections()
+def candidate_connections(device):
+    return nm.get_candidate_connections(device)
 
 
 def set_hosts(*args):
@@ -272,19 +301,23 @@ def set_hosts(*args):
     dns_names = args
 
 
-def assure_hotspot(ssid):
+def assure_hotspot(ssid, device):
     if not nm.get_connection_by_ssid(ssid):
-        nm.make_hotspot(ssid)
+        nm.make_hotspot(ssid, device)
 
 
 def init_states(hosts, callbacks):
+    global hotspot_name
+
     nmmon.init_nmmon()
     set_hosts(*hosts)
 
     for callback in callbacks:
         add_state_callback(callback)
 
-    assure_hotspot(dns_to_conn(hosts[0]))
+    hotspot_name = dns_to_conn(hosts[0])
+
+    assure_hotspot(hotspot_name, modemgr.get_ap_device())
 
 
 def add_state_callback(callback):
